import _asyncToGenerator from '@babel/runtime-corejs3/helpers/asyncToGenerator';
import _includesInstanceProperty from '@babel/runtime-corejs3/core-js/instance/includes';
import _Promise from '@babel/runtime-corejs3/core-js/promise';
import _indexOfInstanceProperty from '@babel/runtime-corejs3/core-js/instance/index-of';
import _JSON$stringify from '@babel/runtime-corejs3/core-js/json/stringify';
import { isAfter, parseISO } from 'date-fns';
import { forEach, map, size } from '@rontrol/lodash';

var SERIALIZED_MARKER = "__lfsc__:";
var SERIALIZED_MARKER_LENGTH = SERIALIZED_MARKER.length;
var TYPE_ARRAYBUFFER = "arbf";
var TYPE_BLOB = "blob";
var TYPE_INT8ARRAY = "si08";
var TYPE_UINT8ARRAY = "ui08";
var TYPE_UINT8CLAMPEDARRAY = "uic8";
var TYPE_INT16ARRAY = "si16";
var TYPE_INT32ARRAY = "si32";
var TYPE_UINT16ARRAY = "ur16";
var TYPE_UINT32ARRAY = "ui32";
var TYPE_FLOAT32ARRAY = "fl32";
var TYPE_FLOAT64ARRAY = "fl64";
var TYPE_SERIALIZED_MARKER_LENGTH = SERIALIZED_MARKER_LENGTH + TYPE_ARRAYBUFFER.length;
var BASE_CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
var BLOB_TYPE_PREFIX = "~~local_forage_type~";
var BLOB_TYPE_PREFIX_REGEX = /^~~local_forage_type~([^~]+)~/;
var toString = Object.prototype.toString;
var localStorageAvailablep = () => {
  var localStorageTestKey = "_localforage_available_test";
  try {
    localStorage.setItem(localStorageTestKey, "true");
    localStorage.removeItem(localStorageTestKey);
    return false;
  } catch (e) {
    return true;
  }
};
var string2Buffer = serializedString => {
  // Fill the string into a ArrayBuffer.
  var bufferLength = serializedString.length * 0.75;
  var len = serializedString.length;
  var i;
  var p = 0;
  var encoded1, encoded2, encoded3, encoded4;
  if (serializedString[serializedString.length - 1] === "=") {
    bufferLength--;
    if (serializedString[serializedString.length - 2] === "=") {
      bufferLength--;
    }
  }
  var buffer = new ArrayBuffer(bufferLength);
  var bytes = new Uint8Array(buffer);
  for (i = 0; i < len; i += 4) {
    encoded1 = /*#__PURE__*/_indexOfInstanceProperty(BASE_CHARS).call(BASE_CHARS, serializedString[i]);
    encoded2 = /*#__PURE__*/_indexOfInstanceProperty(BASE_CHARS).call(BASE_CHARS, serializedString[i + 1]);
    encoded3 = /*#__PURE__*/_indexOfInstanceProperty(BASE_CHARS).call(BASE_CHARS, serializedString[i + 2]);
    encoded4 = /*#__PURE__*/_indexOfInstanceProperty(BASE_CHARS).call(BASE_CHARS, serializedString[i + 3]);

    /*jslint bitwise: true */
    bytes[p++] = encoded1 << 2 | encoded2 >> 4;
    bytes[p++] = (encoded2 & 15) << 4 | encoded3 >> 2;
    bytes[p++] = (encoded3 & 3) << 6 | encoded4 & 63;
  }
  return buffer;
};
var buffer2String = buffer => {
  if (!buffer) {
    return "";
  }
  // base64-arraybuffer
  var bytes = new Uint8Array(buffer);
  var base64String = "";
  var i;
  for (i = 0; i < bytes.length; i += 3) {
    /*jslint bitwise: true */
    base64String += BASE_CHARS[bytes[i] >> 2];
    base64String += BASE_CHARS[(bytes[i] & 3) << 4 | bytes[i + 1] >> 4];
    base64String += BASE_CHARS[(bytes[i + 1] & 15) << 2 | bytes[i + 2] >> 6];
    base64String += BASE_CHARS[bytes[i + 2] & 63];
  }
  if (bytes.length % 3 === 2) {
    base64String = base64String.substring(0, base64String.length - 1) + "=";
  } else if (bytes.length % 3 === 1) {
    base64String = base64String.substring(0, base64String.length - 2) + "==";
  }
  return base64String;
};
var createBlob = (parts, properties) => {
  parts = parts || [];
  properties = properties || {};
  try {
    return new Blob(parts, properties);
  } catch (e) {
    if (e.name !== "TypeError") {
      throw e;
    }
    return;
  }
};
var serialize = arg => {
  var argType = /*#__PURE__*/toString.call(arg);
  if (arg && (argType === "[object ArrayBuffer]" || arg.buffer && toString.call(arg.buffer) === "[object ArrayBuffer]")) {
    var buffer;
    var marker = SERIALIZED_MARKER;
    if (arg instanceof ArrayBuffer) {
      buffer = arg;
      marker += TYPE_ARRAYBUFFER;
    } else {
      buffer = arg.buffer;
      switch (argType) {
        case "[object Int8Array]":
          marker += TYPE_INT8ARRAY;
          break;
        case "[object Uint8Array]":
          marker += TYPE_UINT8ARRAY;
          break;
        case "[object Uint8ClampedArray]":
          marker += TYPE_UINT8CLAMPEDARRAY;
          break;
        case "[object Int16Array]":
          marker += TYPE_INT16ARRAY;
          break;
        case "[object Uint16Array]":
          marker += TYPE_UINT16ARRAY;
          break;
        case "[object Int32Array]":
          marker += TYPE_INT32ARRAY;
          break;
        case "[object Uint32Array]":
          marker += TYPE_UINT32ARRAY;
          break;
        case "[object Float32Array]":
          marker += TYPE_FLOAT32ARRAY;
          break;
        case "[object Float64Array]":
          marker += TYPE_FLOAT64ARRAY;
          break;
        default:
          return _Promise.reject(new Error("Failed to get type for BinaryArray"));
      }
    }
    return _Promise.resolve(marker + buffer2String(buffer));
  } else if (argType === "[object Blob]") {
    // Conver the blob to a binaryArray and then to a string.
    var fileReader = new FileReader();
    fileReader.readAsArrayBuffer(arg);
    var blob2StrPromise = new _Promise((resolve, reject) => {
      fileReader.onload = res => {
        var _res$target;
        // Backwards-compatible prefix for the blob type.
        var str = BLOB_TYPE_PREFIX + (arg === null || arg === void 0 ? void 0 : arg.type) + "~" + buffer2String((_res$target = res.target) === null || _res$target === void 0 ? void 0 : _res$target.result);
        resolve(SERIALIZED_MARKER + TYPE_BLOB + str);
      };
      fileReader.onerror = reject;
    });
    return blob2StrPromise;
  } else {
    try {
      return _Promise.resolve( /*#__PURE__*/_JSON$stringify(arg));
    } catch (e) {
      console.error("Couldn't convert value into a JSON string: ", arg);
      return _Promise.reject(e);
    }
  }
};
var deserialize = arg => {
  // If we haven't marked this string as being specially serialized (i.e.
  // something other than serialized JSON), we can just return it and be
  // done with it.
  if (arg.substring(0, SERIALIZED_MARKER_LENGTH) !== SERIALIZED_MARKER) {
    return JSON.parse(arg);
  }

  // The following code deals with deserializing some kind of Blob or
  // TypedArray. First we separate out the type of data we're dealing
  // with from the data itself.
  var serializedString = /*#__PURE__*/arg.substring(TYPE_SERIALIZED_MARKER_LENGTH);
  var type = /*#__PURE__*/arg.substring(SERIALIZED_MARKER_LENGTH, TYPE_SERIALIZED_MARKER_LENGTH);
  var blobType;
  // Backwards-compatible blob type serialization strategy.
  // DBs created with older versions of localMemory will simply not have the blob type.
  if (type === TYPE_BLOB && BLOB_TYPE_PREFIX_REGEX.test(serializedString)) {
    var _matcher$;
    var matcher = /*#__PURE__*/serializedString.match(BLOB_TYPE_PREFIX_REGEX);
    blobType = matcher === null || matcher === void 0 ? void 0 : matcher[1];
    typeof (matcher === null || matcher === void 0 ? void 0 : (_matcher$ = matcher[0]) === null || _matcher$ === void 0 ? void 0 : _matcher$.length) === "number" && (serializedString = /*#__PURE__*/serializedString.substring(matcher[0].length));
  }
  var buffer = /*#__PURE__*/string2Buffer(serializedString);

  // Return the right type based on the code/type set during
  // serialization.
  switch (type) {
    case TYPE_ARRAYBUFFER:
      return buffer;
    case TYPE_BLOB:
      return createBlob([buffer], {
        type: blobType
      });
    case TYPE_INT8ARRAY:
      return new Int8Array(buffer);
    case TYPE_UINT8ARRAY:
      return new Uint8Array(buffer);
    case TYPE_UINT8CLAMPEDARRAY:
      return new Uint8ClampedArray(buffer);
    case TYPE_INT16ARRAY:
      return new Int16Array(buffer);
    case TYPE_UINT16ARRAY:
      return new Uint16Array(buffer);
    case TYPE_INT32ARRAY:
      return new Int32Array(buffer);
    case TYPE_UINT32ARRAY:
      return new Uint32Array(buffer);
    case TYPE_FLOAT32ARRAY:
      return new Float32Array(buffer);
    case TYPE_FLOAT64ARRAY:
      return new Float64Array(buffer);
    default:
      throw new Error("Unkown type: " + type);
  }
};

var localMemory = _ref => {
  var {
    name = "app"
  } = _ref;
  if (localStorageAvailablep()) {
    throw "localStorage is not supported.";
  }
  var keyPrefix = /*#__PURE__*/"".concat(name, "/");
  var clear = () => {
    for (var i = localStorage.length - 1; i >= 0; i--) {
      var _key = /*#__PURE__*/localStorage.key(i);
      if (_key !== null && _key !== void 0 && _includesInstanceProperty(_key).call(_key, keyPrefix)) {
        localStorage.removeItem(_key);
      }
    }
  };
  var remove = function remove() {
    for (var _len = arguments.length, key = new Array(_len), _key2 = 0; _key2 < _len; _key2++) {
      key[_key2] = arguments[_key2];
    }
    forEach(key, it => {
      localStorage.removeItem(keyPrefix + it);
    });
  };
  var write = /*#__PURE__*/function () {
    var _ref2 = /*#__PURE__*/_asyncToGenerator(function* (key, value) {
      var cracked = yield serialize(value);
      if (cracked) {
        localStorage.setItem(keyPrefix + key, cracked);
      }
      return cracked;
    });
    return function write(_x, _x2) {
      return _ref2.apply(this, arguments);
    };
  }();
  var read = key => {
    var value = /*#__PURE__*/localStorage.getItem(keyPrefix + key);
    if (value) {
      return deserialize(value);
    }
    return;
  };
  var iterate = iterator => {
    var len = localStorage.length;
    for (var i = 0; i < len; i++) {
      var _key3 = /*#__PURE__*/localStorage.key(i);
      if (!(_key3 !== null && _key3 !== void 0 && _includesInstanceProperty(_key3).call(_key3, keyPrefix))) {
        continue;
      }
      var _value = /*#__PURE__*/localStorage.getItem(_key3);

      // If a result was found, parse it from the serialized
      // string into a JS object. If result isn't truthy, the
      // key is likely undefined and we'll pass it straight
      // to the iterator.

      var intact = _value ? deserialize(_value) : undefined;
      iterator(intact, /*#__PURE__*/_key3.substring(keyPrefix.length), i);
    }
  };
  var writeByRecord = arg => {
    var sz = /*#__PURE__*/size(arg);
    if (!sz) {
      return _Promise.resolve();
    }
    return _Promise.all( /*#__PURE__*/map(arg, (v, k) => write(k, v)));
  };
  var has = key => {
    var value = /*#__PURE__*/localStorage.getItem(keyPrefix + key);
    return value !== null;
  };
  var readByExpired = function readByExpired(key) {
    var expireField = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "expireAt";
    var result = /*#__PURE__*/read(key);
    if (result !== null && result !== void 0 && result[expireField]) {
      return isAfter( /*#__PURE__*/parseISO(result[expireField]), new Date()) ? result : undefined;
    }
    return result;
  };
  return {
    clear,
    remove,
    write,
    read,
    readByExpired,
    iterate,
    writeByRecord,
    has,
    keyPrefix,
    name
  };
};

export { localMemory };
